.TH SNMPD 1 "21 Jun 2001"
.ds )H U.C. Davis
.ds ]W VVERSIONINFO
.UC 4
.SH NAME
snmpd - daemon to respond to SNMP request packets.
.SH SYNOPSIS
.B "snmpd [-v] [-a] [-V] [-d] [-q] [-D] [-p [(udp|tcp):]port[@address][,...]] [-x agentx-socket] [-f] [-l logfile] [-L] [-c file] [-C] [-r] [-I [-]module_list]"
.SH DESCRIPTION
.B snmpd
is a SNMP agent which binds to a port and awaits requests from
SNMP management software.  Upon receiving a request, it processes the
request(s), collects the requested information and/or performs the
requested operation(s) and returns the information to the sender.
.SH OPTIONS
.RS
.TP 6
.B -v
Print version information for the agent, and exit.
.TP
.B -a
Dumps the addresses the agent corresponds with to stderr or logfile.
.TP
.B -V
Makes a symbolic dump of the protocol transaction.
.TP
.B -d
Dumps the sent and received UDP packets to stderr or logfile.
.TP 
.B -q
Print simpler output for easier automated parsing.
.TP
.B -D
Turn on debugging output
.TP
.B -p [(udp|tcp):]port[@address][,...]
Makes the agent list on the specified list of sockets instead of the
default port, which is port 161.  Multiple ports can be separated by
commas.  Transports can be specified by prepending the port number
with the transport name ("udp" or "tcp") followed by a colon.
Finally, to bind to a particular interface, you can specify the
address you want it to bind with.
.IP
For example, specifying
.I "-p 161,tcp:161,9161@localhost"
will make the agent listen on: udp port 161 for any address, tcp port
161 for any address, and udp port 9161 on only the interface
associated with the localhost address.  Note that the 
.I "-T"
flag (below) changes the default transport mapping to use (in the
above example, the default transport mapping is 
.I udp.
.TP
.B -x agentx-socket
Listens for AgentX connections on the specified socket
rather than the default '/var/agentx/master'.
The socket can either be a Unix domain socket path,
or the address of a network interface.
If a network address of the form
.B inet-addr:port
is given, then the agent will listen on the port specified.
If a network address of the form
.B inet-addr
is given, then the agent will listen on the default AgentX port, 705.
.TP
.B -f
Don't fork() from the calling shell.
.TP
.B -l logfile
Logs all output from the agent (including stdout/stderr) to 
.I logfile.
Defaults to a compiled option.
.TP
.B -s
Use syslog for logging.
.TP
.B -P pidfile
Save the process ID of the daemon in
.I pidfile.
.TP
.B -g groupid
Change to this gid after opening port.
.TP
.B -u uid
Change to this uid after opening port.
.TP
.B -L
Don't open a log file; use stdout/stderr instead.
.TP
.B -A
Append to the log file rather than truncating it.
.TP
.B -c file
Read 
.I file
as a configuration file.
.TP
.B -C
Don't read any configuration files except the one optionally specified by the 
.B -c 
option.
.TP
.B -r
Don't require root access to run the demon.  Specifically, don't exit
when you can't open files like /dev/kmem, etc...
.TP
.B -I [-]module_list
This option specifies which modules you do (or do not) want to be
initialized when the agent starts up.  If the module_list is preceded
with a '-', it will indicate the modules that you do not want to start
up rather than a definitive list of those that you do want to start.  To get a list of compiled modules, run the agent with the arguments
.I "-Dmib_init -H"
(assumes you have debugging support compiled in).
.RE
.SH CONFIGURATION FILES
.PP
.B snmpd
checks for the existence of and parses the following files:
.TP 6
.B DATADIR/snmp/snmp.conf
Common configuration for the agent and the application. See snmp.conf(5)
for details.
.TP
.B DATADIR/snmp/snmpd.conf
.TP
.B DATADIR/snmp/snmpd.local.conf
Configures the agent.  See snmpd.conf(5) for details.  These files are
optional and are only used to configure the extensible portions of the
agent, the values of the community strings, and the optional trap
destinations.  By default, the first community string ("public" by
default) is allowed read-only access and the second ("private" by
default) is allowed write access as well.  The 3-5th community strings
are read-only as well.
.IP
In addition to these two configuration files in DATADIR/snmp, the
agent will read any files with the names
.I snmpd.conf
and
.I snmpd.local.conf
in a colon separated path specified in the
SNMPCONFPATH environment variable.
.TP
.B DATADIR/snmp/mibs/
The agent will also load all files in this directory as mibs also.  It
will not, however, load any file that begins with a '.' or descend into
subdirectories.
.SH SEE ALSO
(in recommended reading order)
.PP
snmp.conf(5),
snmpd.conf(5)
.\" Local Variables:
.\"  mode: nroff
.\" End:
