/* This file is automatically generated by configure.  Do not modify by hand. */
/* Define if compiling with the ucd_snmp module files.  */
#define USING_UCD_SNMP_MODULE 1
 
/* Define if compiling with the snmpv3mibs module files.  */
#define USING_SNMPV3MIBS_MODULE 1
 
/* Define if compiling with the notification module files.  */
#define USING_NOTIFICATION_MODULE 1
 
/* Define if compiling with the agentx module files.  */
//#define USING_AGENTX_MODULE 1
 
/* Define if compiling with the smux module files.  */
#define USING_SMUX_MODULE 1
 
/* Define if compiling with the target module files.  */
#define USING_TARGET_MODULE 1
 
/* Define if compiling with the examples/ucdDemoPublic module files.  */
#define USING_EXAMPLES_UCDDEMOPUBLIC_MODULE 1
 
/* Define if compiling with the examples/example module files.  */
#define USING_EXAMPLES_EXAMPLE_MODULE 1
 
/* Define if compiling with the mibII/system_mib module files.  */
#define USING_MIBII_SYSTEM_MIB_MODULE 1
 
/* Define if compiling with the mibII/vacm_vars module files.  */
#define USING_MIBII_VACM_VARS_MODULE 1
 
/* Define if compiling with the mibII/snmp_mib module files.  */
#define USING_MIBII_SNMP_MIB_MODULE 1
 
/* Define if compiling with the mibII/sysORTable module files.  */
#define USING_MIBII_SYSORTABLE_MODULE 1
 
/* Define if compiling with the ucd-snmp/proc module files.  */
#define USING_UCD_SNMP_PROC_MODULE 1
 
/* Define if compiling with the ucd-snmp/versioninfo module files.  */
#define USING_UCD_SNMP_VERSIONINFO_MODULE 1
 
/* Define if compiling with the ucd-snmp/pass module files.  */
#define USING_UCD_SNMP_PASS_MODULE 1
 
/* Define if compiling with the ucd-snmp/pass_persist module files.  */
#define USING_UCD_SNMP_PASS_PERSIST_MODULE 1
 
/* Define if compiling with the ucd-snmp/disk module files.  */
#define USING_UCD_SNMP_DISK_MODULE 1
 
/* Define if compiling with the ucd-snmp/loadave module files.  */
#define USING_UCD_SNMP_LOADAVE_MODULE 1
 
/* Define if compiling with the ucd-snmp/extensible module files.  */
#define USING_UCD_SNMP_EXTENSIBLE_MODULE 1
 
/* Define if compiling with the ucd-snmp/errormib module files.  */
#define USING_UCD_SNMP_ERRORMIB_MODULE 1
 
/* Define if compiling with the ucd-snmp/file module files.  */
#define USING_UCD_SNMP_FILE_MODULE 1
 
/* Define if compiling with the ucd-snmp/dlmod module files.  */
#define USING_UCD_SNMP_DLMOD_MODULE 1
 
/* Define if compiling with the ucd-snmp/proxy module files.  */
#define USING_UCD_SNMP_PROXY_MODULE 1
 
/* Define if compiling with the snmpv3/snmpEngine module files.  */
#define USING_SNMPV3_SNMPENGINE_MODULE 1
 
/* Define if compiling with the snmpv3/snmpMPDStats module files.  */
#define USING_SNMPV3_SNMPMPDSTATS_MODULE 1
 
/* Define if compiling with the snmpv3/usmStats module files.  */
#define USING_SNMPV3_USMSTATS_MODULE 1
 
/* Define if compiling with the snmpv3/usmUser module files.  */
#define USING_SNMPV3_USMUSER_MODULE 1
 
/* Define if compiling with the notification/snmpNotifyTable module files.  */
#define USING_NOTIFICATION_SNMPNOTIFYTABLE_MODULE 1
 
/* Define if compiling with the notification/snmpNotifyFilterTable module files.  */
#define USING_NOTIFICATION_SNMPNOTIFYFILTERTABLE_MODULE 1
 
/* Define if compiling with the notification/snmpNotifyFilterProfileTable module files.  */
#define USING_NOTIFICATION_SNMPNOTIFYFILTERPROFILETABLE_MODULE 1
 
/* Define if compiling with the agentx/master module files.  */
//#define USING_AGENTX_MASTER_MODULE 1
 
/* Define if compiling with the agentx/subagent module files.  */
//#define USING_AGENTX_SUBAGENT_MODULE 1
 
/* Define if compiling with the smux/smux module files.  */
#define USING_SMUX_SMUX_MODULE 1
 
/* Define if compiling with the target/snmpTargetAddrEntry module files.  */
#define USING_TARGET_SNMPTARGETADDRENTRY_MODULE 1
 
/* Define if compiling with the target/snmpTargetParamsEntry module files.  */
#define USING_TARGET_SNMPTARGETPARAMSENTRY_MODULE 1
 
/* Define if compiling with the target/target module files.  */
#define USING_TARGET_TARGET_MODULE 1
 
/* Define if compiling with the util_funcs module files.  */
#define USING_UTIL_FUNCS_MODULE 1
 
/* Define if compiling with the header_complex module files.  */
#define USING_HEADER_COMPLEX_MODULE 1
 
/* Define if compiling with the agentx/protocol module files.  */
//#define USING_AGENTX_PROTOCOL_MODULE 1
 
/* Define if compiling with the agentx/client module files.  */
//#define USING_AGENTX_CLIENT_MODULE 1
 
/* Define if compiling with the agentx/master_admin module files.  */
//#define USING_AGENTX_MASTER_ADMIN_MODULE 1
 
/* Define if compiling with the agentx/master_request module files.  */
//#define USING_AGENTX_MASTER_REQUEST_MODULE 1
 
/* Define if compiling with the mibII/udp module files.  */
#define USING_MIBII_UDP_MODULE 1

/* Define if compiling with the mibII/interfaces module files.  */
#define USING_MIBII_INTERFACES_MODULE 1

/* Define if compiling with the mibII/at module files.  */
#define USING_MIBII_AT_MODULE 1

/* Define if compiling with the mibII/ip module files.  */
#define USING_MIBII_IP_MODULE 1

/* Define if compiling with the mibII/icmp module files.  */
#define USING_MIBII_ICMP_MODULE 1

/* Define if compiling with the mibII/tcp module files.  */
#define USING_MIBII_TCP_MODULE 1
