#ifdef UCD_COMPATIBLE

#include <net-snmp/version.h>

static const char *VersionInfo = NetSnmpVersionInfo;

#else

#error "Please update your headers or configure using --enable-ucd-snmp-compatibility"

#endif
