/* A simple helpful wrapper to include lots of agent specific include
 * files for people wanting to embed and snmp agent into an external
 * application
 */
#ifndef UCD_SNMP_AGENT_INCLUDES_H
#define UCD_SNMP_AGENT_INCLUDES_H

#include <ucd-snmp/snmp_agent.h>
#include <ucd-snmp/snmp_vars.h>
#include <ucd-snmp/ds_agent.h>
#include <ucd-snmp/var_struct.h>
#include <ucd-snmp/agent_read_config.h>

#endif /* UCD_SNMP_AGENT_INCLUDES_H */
