.\" /***********************************************************
.\" 	Copyright 1988, 1989 by Carnegie Mellon University
.\" 
.\"                       All Rights Reserved
.\" 
.\" Permission to use, copy, modify, and distribute this software and its 
.\" documentation for any purpose and without fee is hereby granted, 
.\" provided that the above copyright notice appear in all copies and that
.\" both that copyright notice and this permission notice appear in 
.\" supporting documentation, and that the name of CMU not be
.\" used in advertising or publicity pertaining to distribution of the
.\" software without specific, written prior permission.  
.\" 
.\" CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
.\" ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
.\" CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
.\" ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
.\" WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
.\" ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
.\" SOFTWARE.
.\" ******************************************************************/
.TH SNMPSTATUS 1 "08 Feb 2002" VVERSIONINFO "Net-SNMP"
.UC 4
.SH NAME
snmpstatus - Retrieves important information from a network entity.
.SH SYNOPSIS
.B snmpstatus
[COMMON OPTIONS]
.SH DESCRIPTION
.B snmpstatus
is an SNMP application that retrieves several important statistics
from a network entity.
.PP
The information returned is:
.IP
The IP address of the entity.
.br
A textual description of the entity (sysDescr.0)
.br
The uptime of the entity's SNMP agent (sysUpTime.0)
.br
The sum of received packets on all interfaces
(ifInUCastPkts.* + ifInNUCastPkts.*)
.br
The sum of transmitted packets on all interfaces
(ifOutUCastPkts.* + ifOutNUCastPkts.*)
.br
The number of IP input packets (ipInReceives.0)
.br
The number of IP output packets (ipOutRequests.0)
.PP
For example:
.PP
snmpstatus -c public -v 1 netdev-kbox.cc.cmu.edu
.PP
will produce output similar to the following:
.PP
[128.2.56.220]=>[Kinetics FastPath2] Up: 1 day, 4:43:31
.br
Interfaces: 1,  Recv/Trans packets: 262874/39867 |
IP: 31603/15805
.PP
.B snmpstatus
also checks the operational status of all interfaces (ifOperStatus.*),
and if it finds any that are not running, it will report in a manner
similar to this:
.PP
2 interfaces are down!
.PP
If the network entity has an error processing the request packet, an
error packet will be returned and a message will be shown, helping to
pinpoint in what way the request was malformed.
.B snmpstatus
will attempt to reform its request to eliminate the malformed
variable (unless the
.B -Cf
option is given, see below), but this variable will then be missing
from the displayed data.
.PP
.SH OPTIONS
.B snmpstatus
takes the common options described in the
.I snmpcmd(1)
manual page and
also the 
.B -Cf
option described in the 
.I snmpget(1)
manual page.
.SH "SEE ALSO"
snmpcmd(1), snmpget(1)
